@@ -40,9 +40,6 @@ extern const VADisplayHooks va_display_hooks_win32;
 
 static const VADisplayHooks *g_display_hooks;
 static const VADisplayHooks *g_display_hooks_available[] = {
-#ifdef ANDROID
-    &va_display_hooks_android,
-#else
 #ifdef HAVE_VA_WIN32
     &va_display_hooks_win32,
 #endif
@@ -55,9 +52,7 @@ static const VADisplayHooks *g_display_hooks_available[] = {
 #ifdef HAVE_VA_DRM
     &va_display_hooks_drm,
 #endif
-#endif
-    NULL
-};
+    NULL};
 
 static const char *g_display_name;
 const char *g_device_name;
