@@ -80,6 +80,13 @@
         exit(1);                                                        \
     }
 
+#define CHECK_CONDITION(cond)                                                \
+    if(!(cond))                                                              \
+    {                                                                        \
+        fprintf(stderr, "Unexpected condition: %s:%d\n", __func__, __LINE__); \
+        exit(1);                                                             \
+    }
+
 static VADisplay va_dpy;
 
 static int picture_width, picture_width_in_mbs;
@@ -585,9 +592,10 @@ static void upload_yuv_to_surface(FILE *yuv_fp, VASurfaceID surface_id)
     u_src = newImageBuffer + y_size; /* UV offset for NV12 */
     v_src = newImageBuffer + y_size + u_size;
 
-    y_dst = surface_p + surface_image.offsets[0];
-    u_dst = surface_p + surface_image.offsets[1]; /* UV offset for NV12 */
-    v_dst = surface_p + surface_image.offsets[2];
+    y_dst = (unsigned char *)surface_p + surface_image.offsets[0];
+    u_dst = (unsigned char *)surface_p +
+            surface_image.offsets[1]; /* UV offset for NV12 */
+    v_dst = (unsigned char *)surface_p + surface_image.offsets[2];
 
     /* Y plane */
     for (row = 0; row < surface_image.height; row++) {
@@ -612,8 +620,8 @@ static void upload_yuv_to_surface(FILE *yuv_fp, VASurfaceID surface_id)
         const int U = surface_image.format.fourcc == VA_FOURCC_I420 ? 1 : 2;
         const int V = surface_image.format.fourcc == VA_FOURCC_I420 ? 2 : 1;
 
-        u_dst = surface_p + surface_image.offsets[U];
-        v_dst = surface_p + surface_image.offsets[V];
+        u_dst = (unsigned char *)surface_p + surface_image.offsets[U];
+        v_dst = (unsigned char *)surface_p + surface_image.offsets[V];
 
         for (row = 0; row < surface_image.height / 2; row++) {
             memcpy(u_dst, u_src, surface_image.width / 2);
@@ -848,7 +856,7 @@ static int begin_picture(FILE *yuv_fp, int frame_num, int display_num, int slice
     /* hrd parameter */
     VAEncMiscParameterBuffer *misc_param;
     VAEncMiscParameterHRD *misc_hrd_param;
-    vaCreateBuffer(va_dpy,
+    va_status = vaCreateBuffer(va_dpy,
                    avcenc_context.context_id,
                    VAEncMiscParameterBufferType,
                    sizeof(VAEncMiscParameterBuffer) + sizeof(VAEncMiscParameterRateControl),
@@ -878,13 +886,14 @@ static int begin_picture(FILE *yuv_fp, int frame_num, int display_num, int slice
         VAEncMiscParameterBufferROI *misc_roi_param;
 
         int roi_num = 1;
-        vaCreateBuffer(va_dpy,
+        va_status = vaCreateBuffer(va_dpy,
                        avcenc_context.context_id,
                        VAEncMiscParameterBufferType,
                        sizeof(VAEncMiscParameterBuffer) + sizeof(VAEncMiscParameterBufferROI) + roi_num * sizeof(VAEncROI),
                        1,
                        NULL,
                        &avcenc_context.misc_parameter_roi_buf_id);
+        CHECK_VASTATUS(va_status, "vaCreateBuffer");
         vaMapBuffer(va_dpy,
                     avcenc_context.misc_parameter_roi_buf_id,
                     (void **)&misc_param);
@@ -1792,7 +1801,8 @@ encode_picture(FILE *yuv_fp, FILE *avc_fp,
             index = SID_INPUT_PICTURE_0;
         if (next_display_num >= frame_number)
             next_display_num = frame_number - 1;
-        fseeko(yuv_fp, (off_t)frame_size * next_display_num, SEEK_SET);
+        ret = fseeko(yuv_fp, (off_t)frame_size * next_display_num, SEEK_SET);
+        CHECK_CONDITION(ret == 0);
 
         avcenc_context.upload_thread_param.yuv_fp = yuv_fp;
         avcenc_context.upload_thread_param.surface_id = surface_ids[index];
@@ -2108,6 +2118,11 @@ int main(int argc, char *argv[])
     file_size = ftello(yuv_fp);
     frame_size = picture_width * picture_height + ((picture_width * picture_height) >> 1) ;
 
+    if (frame_size == 0) {
+        fclose(yuv_fp);
+        printf("Frame size is not correct\n");
+        return -1;
+    }
     if ((file_size < frame_size) || (file_size % frame_size)) {
         fclose(yuv_fp);
         printf("The YUV file's size is not correct\n");
