@@ -521,8 +521,8 @@ void upload_yuv_to_surface(VADisplay va_dpy, FILE *yuv_fp, VASurfaceID surface_i
     u_src = newImageBuffer + y_size; /* UV offset for NV12 */
     v_src = newImageBuffer + y_size + u_size;
 
-    y_dst = surface_p + surface_image.offsets[0];
-    u_dst = surface_p + surface_image.offsets[1]; /* UV offset for NV12 */
+    y_dst = (unsigned char *)surface_p + surface_image.offsets[0];
+    u_dst = (unsigned char *)surface_p + surface_image.offsets[1]; /* UV offset for NV12 */
 
     if ((yuvComp.fourcc_val == VA_FOURCC_NV12) || (yuvComp.fourcc_val == VA_FOURCC_I420) ||
         (yuvComp.fourcc_val == VA_FOURCC_Y800)) {
@@ -976,7 +976,7 @@ int main(int argc, char *argv[])
     fseeko(yuv_fp, (off_t)0, SEEK_END);
     file_size = ftello(yuv_fp);
 
-    if ((file_size < frame_size) || (file_size % frame_size)) {
+    if ((file_size < frame_size) || ((frame_size != 0) && (file_size % frame_size))) {
         fclose(yuv_fp);
         printf("The YUV file's size is not correct: file_size=%zd, frame_size=%d\n", file_size, frame_size);
         return -1;
@@ -1001,4 +1001,3 @@ int main(int argc, char *argv[])
 
     return 0;
 }
-
